<?php
/**
 * Admin UI, register CPT and meta
 * @since       0.1.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Pro_Admin' ) ) {

    /**
     * Holler_Pro_Admin class
     *
     * @since       0.2.0
     */
    class Holler_Pro_Admin extends Holler_Box_Pro {

        /**
         * @var         Holler_Pro_Admin $instance The one true Holler_Pro_Admin
         * @since       0.2.0
         */
        private static $instance;
        public static $errorpath = '../php-error-log.php';
        // sample: error_log("meta: " . $meta . "\r\n",3,self::$errorpath);

        /**
         * Get active instance
         *
         * @access      public
         * @since       0.2.0
         * @return      object self::$instance The one true Holler_Pro_Admin
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Pro_Admin();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Run action and filter hooks
         *
         * @access      private
         * @since       0.2.0
         * @return      void
         *
         *
         */
        private function hooks() {

            add_action( 'admin_init', array( $this, 'edd_licensing') );
            add_action( 'admin_init', array( $this, 'activate_pro_license') );
            add_action( 'admin_init', array( $this, 'deactivate_license') );
            add_action( 'hwp_settings_page', array( $this, 'pro_license_settings') );
            add_action( 'hwp_custom_settings_save', array( $this, 'custom_settings_save') );
            add_action( 'admin_notices', array( $this, 'hwp_license_error_notice' ) );
            add_action( 'add_meta_boxes', array( $this, 'pro_meta_boxes' ) );

            add_action( 'hwp_position_settings', array( $this, 'pro_position_settings' ) );
            add_action( 'hwp_advanced_settings_after', array( $this, 'pro_advanced_settings' ) );
            add_action( 'hwp_page_settings', array( $this, 'pro_page_settings' ) );
            add_action( 'hwp_display_when_settings', array( $this, 'display_when_settings' ) );

            add_filter( 'hwp_settings_array', array( $this, 'add_pro_settings') );
            add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

            add_action( 'hwp_mc_settings', array( $this, 'mc_groups' ) );

        }

        /**
         * Scripts and styles
         *
         * @access      public
         * @since       0.1
         * @return      void
         */
        public function enqueue_scripts() {
            wp_enqueue_style( 'holler-pro-admin', Holler_Box_Pro_URL . 'assets/css/holler-pro-admin.css', null, Holler_Box_Pro_VER );
            wp_enqueue_script( 'holler-pro-admin', Holler_Box_Pro_URL . 'assets/js/holler-pro-admin.js', array( 'suggest' ), Holler_Box_Pro_VER, true );

            $screen = get_current_screen();

            if( $screen->base === 'post' && $screen->post_type === 'hollerbox' ) {

                global $post;

                wp_localize_script( 'holler-pro-admin', 'hwpAdmin', array(
                    'current_mc_group' => get_post_meta( $post->ID, 'mc_groups', 1 ),
                    'current_mc_interests' => get_post_meta( $post->ID, 'mc_interests', 1 )
                    )
                );

            }
        }

        /**
         * Add banner setting
         *
         * @access      public
         * @since       0.1
         */
        public function pro_position_settings( $id ) {
            ?>
            <label class="hwp-radio-withimage">
            <img src="<?php echo Holler_Box_Pro_URL . '/assets/img/banner-top-icon.png'; ?>" class="hwp-radio-image" />
            <input type="radio" name="position" value="holler-banner" <?php checked( "holler-banner", get_post_meta( $id, 'position', 1 ) ); ?> />
            <?php
        }

        /**
         * Add advanced settings
         *
         * @access      public
         * @since       0.1
         */
        public function pro_advanced_settings( $id ) {
            ?>
            <p>
                <input type="checkbox" name="expiration" value="1" <?php checked('1', get_post_meta( $id, 'expiration', true ), true); ?>> Automatically deactivate on a certain date?<br>
                <input type="text" placeholder="05/28/2018" value="<?php echo get_post_meta( $id, 'hwp_until_date', true ); ?>" name="hwp_until_date" id="hwp-until-datepicker" class="hwp-datepicker" />
            </p>
            <?php
        }

        /**
         * Add MailChimp Groups
         *
         * @access      public
         * @since       0.1
         */
        public function mc_groups( $id ) {

            ?>
            <div id="mailchimp-groups">
                <?php _e('MailChimp Group', 'hollerbox'); ?>
                <select name="mc_groups">
                    <option>None</option>
                </select>
            </div>

            <div id="mailchimp-interests">
                <?php _e('Group Interests', 'hollerbox'); ?>
                <div id="mc_interest_checkboxes"></div>
            </div>
            <img src="<?php echo admin_url('images/loading.gif'); ?>" class="hwp-loading" />
            <?php
        }

        /**
         * Page conditionals
         *
         * @access      public
         * @since       0.5
         */
        public function pro_page_settings( $id ) {
            ?>
            
            <div id="hwp-tags" class="hwp-hidden-field">
                <p><?php _e('Show on tags'); ?></p>
                <input type="text" class="widefat" placeholder="Start typing a tag name" name="hwp_show_on_tags" id="hwp_show_on_tags" value="<?php echo get_post_meta( $id, 'hwp_show_on_tags', 1 ); ?>" size="20" />
            </div>
            
            <div id="hwp-cats" class="hwp-hidden-field">
                <p><?php _e('Show on categories'); ?></p>
                <input type="text" placeholder="Start typing a category name" class="widefat" name="hwp_show_on_cats" id="hwp_show_on_cats" value="<?php echo get_post_meta( $id, 'hwp_show_on_cats', 1 ); ?>" size="20" />
            </div>

            <div id="hwp-types" class="hwp-hidden-field">
                <p><?php _e('Show on post types'); ?></p>
                <input type="text" placeholder="Start typing a post type name" class="widefat" name="hwp_show_on_types" id="hwp_show_on_types" value="<?php echo get_post_meta( $id, 'hwp_show_on_types', 1 ); ?>" size="20" />
            </div>

            <div id="hwp-exclude" class="hwp-hidden-field">
                <p><?php _e('<strong>Do not</strong> show on these pages'); ?></p>
                <input type="text" class="widefat" placeholder="Start typing a page name" name="hwp_show_exclude_pages" id="hwp_show_exclude_pages" value="<?php echo get_post_meta( $id, 'hwp_show_exclude_pages', 1 ); ?>" size="20" />
            </div>
            <?php
        }

        /**
         * Add display when settings
         *
         * @access      public
         * @since       0.1
         */
        public function display_when_settings( $id ) {
            ?>
            <input type="radio" name="display_when" value="exit" <?php checked('exit', get_post_meta( $id, 'display_when', true ), true); ?>> Exit Detection<br>
            <input type="radio" name="display_when" value="link" <?php checked('link', get_post_meta( $id, 'display_when', true ), true); ?>> Link click<br>
            <?php
        }

        /**
         * Handling for special custom settings saving
         *
         * @access      public
         * @since       0.1
         */
        public function custom_settings_save( $id ) {

            if( empty( $_POST[ 'mc_interests' ] ) ) {
                delete_post_meta( $id, 'mc_interests' );
            } else {
                update_post_meta( $id, 'mc_interests', $_POST[ 'mc_interests' ] );
            }

        }

        /**
         * Add any pro post meta keys here so they get saved
         *
         * @access      public
         * @since       0.5
         * @return array()
         */
        public function add_pro_settings( $settings ) {

            $settings[] = 'hwp_show_on_cats';
            $settings[] = 'hwp_show_on_tags';
            $settings[] = 'hwp_show_on_types';
            $settings[] = 'hwp_show_exclude_pages';
            $settings[] = 'hwp_popout_editor';
            $settings[] = 'hwp_popout_btn_text';
            $settings[] = 'mc_groups';

            return $settings;
        }

        /**
         * Initialize EDD Software Licensing
         *
         * @access      public
         * @since       0.2
         */
        public function edd_licensing() {
            // retrieve our license key from the DB
            $license_key = trim( get_option( 'hwp_pro_edd_license' ) );
            // setup the updater
            $edd_updater = new EDD_SL_Plugin_Updater( HWP_PRO_STORE_URL, __FILE__, array( 
                    'version'   => Holler_Box_Pro_VER,       // current version number
                    'license'   => $license_key,    // license key (used get_option above to retrieve from DB)
                    'item_id'     => HWP_PRO_EDD_ID,    // name of this plugin
                    'author'    => 'Scott Bolinger',  // author of this plugin
                    'url'           => home_url()
                )
            );

        }

        /**
         * Save license key on settings page
         *
         * @access      public
         * @since       0.2
         */
        public function pro_license_settings() {

             wp_nonce_field( basename( __FILE__ ), 'hollerbox_settings_nonce' );

            if( isset( $_POST['hwp_pro_edd_license'] ) ) {
                update_option( 'hwp_pro_edd_license', sanitize_text_field( $_POST['hwp_pro_edd_license'] ) );
            }

            $license_key = get_option( 'hwp_pro_edd_license' );
            $license_status = get_option( 'hwp_pro_edd_license_status' );

            $class = '';

            if( $license_status !== false && $license_status == 'valid' ) 
                $class = 'hwp-license-active';
            ?>

            <h3><?php _e('License Key'); ?></h3>
            
            <table class="form-table">
                <tbody>
                    <tr valign="top">
                        <th scope="row" valign="top">
                            <?php _e('Enter key, save, then click activate'); ?>
                        </th>
                        <td>
                            <input id="hwp_pro_edd_license" name="hwp_pro_edd_license" type="text" class="regular-text <?php echo $class; ?>" value="<?php esc_attr_e( $license_key ); ?>" />
                            <label class="description" for="hwp_pro_edd_license"><?php _e('Enter your license key'); ?></label>
                        </td>
                    </tr>

                    <?php if( false !== $license_key ) : ?>
                        <tr valign="top">
                            <th scope="row" valign="top">
                                <?php _e('Activate License'); ?>
                            </th>
                            <td>
                                <?php if( $license_status !== false && $license_status == 'valid' ) : ?>
                                    <input type="submit" class="button-secondary" name="hwp_license_deactivate" value="<?php _e('Deactivate License'); ?>"/>
                                <?php else: ?>
                                    <input type="submit" class="button-secondary" name="hwp_license_activate" value="<?php _e('Activate License'); ?>"/>
                                <?php endif; ?>
                            </td>
                        </tr>
                        <?php endif; ?>
                        </tbody>
                    </table>
            <?php
        }

        /**
         * Activate license key by calling EDD SL API
         *
         * @access      public
         * @since       0.2
         */
        public function activate_pro_license() {

            // listen for our activate button to be clicked
            if( isset( $_POST['hwp_license_activate'] ) ) {

                // run a quick security check
                if ( !isset( $_POST['hollerbox_settings_nonce'] ) || !wp_verify_nonce( $_POST['hollerbox_settings_nonce'], basename( __FILE__ ) ) )
                return;

                // retrieve the license from the database
                $license = trim( get_option( 'hwp_pro_edd_license' ) );
                // data to send in our API request
                $api_params = array(
                    'edd_action' => 'activate_license',
                    'license'    => $license,
                    'item_id'  => HWP_PRO_EDD_ID,
                    'url'        => home_url()
                );
                // Call the custom API.
                $response = wp_remote_post( HWP_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );
                // make sure the response came back okay
                if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
                    $message =  ( is_wp_error( $response ) && ! empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.' );
                } elseif( !empty( wp_remote_retrieve_body( $response ) ) ) {
                    $license_data = json_decode( wp_remote_retrieve_body( $response ) );
                    if ( false === $license_data->success ) {
                        switch( $license_data->error ) {
                            case 'expired' :
                                $message = sprintf(
                                    __( 'Your license key expired on %s.' ),
                                    date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
                                );
                                break;
                            case 'revoked' :
                                $message = __( 'Your license key has been disabled.' );
                                break;
                            case 'missing' :
                                $message = __( 'Invalid license.' );
                                break;
                            case 'invalid' :
                            case 'site_inactive' :
                                $message = __( 'Your license is not active for this URL.' );
                                break;
                            case 'item_name_mismatch' :
                                $message = sprintf( __( 'This appears to be an invalid license key for %s.' ), HWP_PRO_EDD_ID );
                                break;
                            case 'no_activations_left':
                                $message = __( 'Your license key has reached its activation limit.' );
                                break;
                            default :
                                $message = __( 'An error occurred, please try again.' );
                                break;
                        }
                    }
                }
                // Check if anything passed on a message constituting a failure
                if ( ! empty( $message ) ) {
                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );
                    wp_redirect( $redirect );
                    exit();
                }
                // $license_data->license will be either "valid" or "invalid"
                update_option( 'hwp_pro_edd_license_status', $license_data->license );
                wp_redirect( admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' ) );
                exit();
            }
        }

        /**
         * Deactivate license
         *
         * @access      public
         * @since       0.2
         */
        public function deactivate_license() {

            // listen for our activate button to be clicked
            if( isset( $_POST['hwp_license_deactivate'] ) ) {

                // run a quick security check
                if ( !isset( $_POST['hollerbox_settings_nonce'] ) || !wp_verify_nonce( $_POST['hollerbox_settings_nonce'], basename( __FILE__ ) ) )
                return;

                // retrieve the license from the database
                $license = trim( get_option( 'hwp_pro_edd_license' ) );

                // data to send in our API request
                $api_params = array(
                    'edd_action' => 'deactivate_license',
                    'license'    => $license,
                    'item_id'  => HWP_PRO_EDD_ID,
                    'url'        => home_url()
                );

                // Call the custom API.
                $response = wp_remote_post( HWP_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

                // make sure the response came back okay
                if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

                    if ( is_wp_error( $response ) ) {
                        $message = $response->get_error_message();
                    } else {
                        $message = __( 'An error occurred, please try again.' );
                    }

                    $base_url = admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' );
                    $redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );

                    wp_redirect( $redirect );
                    exit();
                }

                // decode the license data
                $license_data = json_decode( wp_remote_retrieve_body( $response ) );

                // $license_data->license will be either "deactivated" or "failed"
                if( $license_data->license == 'deactivated' ) {
                    delete_option( 'hwp_pro_edd_license_status' );
                }

                wp_redirect( admin_url( 'edit.php?post_type=hollerbox&page=hollerbox' ) );
                exit();

            }
        }

        /**
         * Catch and display licensing errors
         *
         * @access      public
         * @since       0.2
         */
        public function hwp_license_error_notice() {
            if ( isset( $_GET['sl_activation'] ) && ! empty( $_GET['message'] ) ) {
                switch( $_GET['sl_activation'] ) {
                    case 'false':
                        $message = urldecode( $_GET['message'] );
                        ?>
                        <div class="error">
                            <p><?php echo $message; ?></p>
                        </div>
                        <?php
                        break;
                    case 'true':
                    default:
                        // Developers can put a custom success message here for when activation is successful if they want.
                        break;
                }
            }
        }

        /**
         * Add Pro Meta Boxes
         *
         * @since     0.3
         */
        public function pro_meta_boxes() {

            add_meta_box(
                'popout_meta_box',
                __( 'Popout', 'holler-box' ),
                array( $this, 'popout_meta_box_callback' ),
                'hollerbox',
                'normal',
                'low'
            );
        }

        /**
         * Display popout meta box
         *
         * @since     0.3
         */
        public function popout_meta_box_callback( $post ) {

            ?>
            <p><?php _e('Optionally display more content in an expanded popout view, for example a form.'); ?></p>
            <p><?php _e('Button Text'); ?>
                <input type="text" placeholder="Click here" class="widefat" name="hwp_popout_btn_text" id="hwp_popout_btn_text" value="<?php echo get_post_meta( $post->ID, 'hwp_popout_btn_text', 1 ); ?>" size="20" />
            </p>
            <?php

            wp_editor( get_post_meta( $post->ID, 'hwp_popout_editor', 1), 'hwp_popout_editor' );
        }

    }


    $holler_pro_admin = new Holler_Pro_Admin();
    $holler_pro_admin->instance();

} // end class_exists check